#pragma once
#include <vector>

#include "fmod.hpp"
#include "fmod_errors.h"
using namespace FMOD;

#pragma comment(lib,"fmodex_vc.lib")

class Game
{
	// Game Components:
	
FMOD_RESULT result;
FMOD::System *system;
FMOD::Channel *channel;
FMOD::Channel *musicChannel;
FMOD::Channel *titleChannel;
FMOD::Sound *titleMusic;
FMOD::Sound *menuBlip;
FMOD::Sound *gunShot;
FMOD::Sound *message;
FMOD::Sound *explosion;
FMOD::Sound *number;
FMOD::Sound *bgMusic;
FMOD::Sound *win;
FMOD::Sound *hit;

int MapSelection;

	// Window Handle:
	HWND hWnd;

	// Variables used to perform back buffering:
	HDC hdcDisplay;
	HDC hdcMemory;
	HBITMAP hbmBackBuffer;
	HBITMAP hbmOld;
	RECT rClient;

	// Variables used to calculate frames per second:
	DWORD dwFrames;
	DWORD dwCurrentTime;
	DWORD dwLastUpdateTime;
	DWORD dwElapsedTime;
	TCHAR szFPS[32];


	//// Variables used for display the TitleMenu
	HDC hdcTitle;
	HBITMAP hbmTitle;
	HBITMAP hbmTitleOld;
	BITMAP bmTitleInfo;

	//// Variables used for display the MapSelection
	HDC hdcTitleMap;
	HBITMAP hbmTitleMap;
	HBITMAP hbmTitleMapOld;
	BITMAP bmTitleMapInfo;

	// Variables used for display the score
	HDC hdcScore;
	HBITMAP hbmScore;
	HBITMAP hbmScoreOld;
	BITMAP bmScoreInfo;

	// Variables used for display the Level bitmap:
	HDC hdcLevel;
	HBITMAP hbmLevel;
	HBITMAP hbmLevelOld;
	BITMAP bmLevelInfo;

	// Variables used for display the Bomb bitmap:
	HDC hdcBomb;
	HBITMAP hbmBomb;
	HBITMAP hbmBombOld;
	BITMAP bmBombInfo;
	int BombX;
	int BombY;
	bool BombHasBeenPlanted;
	RECT BombBox;

	// Variables used for display the Player1 bitmap:
	HDC hdcPlayer1;
	HBITMAP hbmPlayer1;
	HBITMAP hbmPlayer1Old;
	BITMAP bmPlayer1Info;

	// Variables used for display the Player1 HUD:
	HDC hdcPlayer1HUD;
	HBITMAP hbmPlayer1HUD;
	HBITMAP hbmPlayer1HUDOld;
	BITMAP bmPlayer1HUDInfo;

	
	// Variables used for display the Player2 bitmap
	HDC hdcPlayer2;
	HBITMAP hbmPlayer2;
	HBITMAP hbmPlayer2Old;
	BITMAP bmPlayer2Info;

	// Variables used for display the Player2 HUD:
	HDC hdcPlayer2HUD;
	HBITMAP hbmPlayer2HUD;
	HBITMAP hbmPlayer2HUDOld;
	BITMAP bmPlayer2HUDInfo;


	// Variables used for display the TargetArrow
	HDC hdcTargetArrow;
	HBITMAP hbmTargetArrow;
	HBITMAP hbmTargetArrowOld;
	BITMAP bmTargetArrowInfo;
	int TargetArrowX;
	int TargetArrowY;

	// Variables used for display the Player1 Target
	HDC hdcPlayer1Target;
	HBITMAP hbmPlayer1Target;
	HBITMAP hbmPlayer1TargetOld;
	BITMAP bmPlayer1TargetInfo;
	int Target1X;
	int Target1Y;
	RECT Target1;

	// Variables used for display the Player2 Target
	HDC hdcPlayer2Target;
	HBITMAP hbmPlayer2Target;
	HBITMAP hbmPlayer2TargetOld;
	BITMAP bmPlayer2TargetInfo;
	int Target2X;
	int Target2Y;
	RECT Target2;


	//Variables for bombplant time
	int BombTime;
	
	char textState;//DISPLAY INFO BASED ON STATE OF BOMB
	
	int size;
	int timer;
	int alpha; //ALPHA VALUE FOR TEXT

	//PENS/BRUSH
	/*HPEN pen;
	HPEN oldpen;*/

	//player 1
	char Player1direction;
	int Player1Frame;
	int Player1X;
	int Player1Y;
	int Player1Health;
	int Player1Shield;
	int Player1Score;
	bool p1KeyPressed;
	bool p1HasBomb;
	bool p1AtTarget2;
    RECT P1;

	//player 2
	char Player2direction;
	int Player2Frame;
	int Player2X;
	int Player2Y;
	int Player2Health;
	int Player2Shield;
	int Player2Score;
	bool p2KeyPressed;
	bool p2HasBomb;
	bool p2AtTarget1;
	RECT P2;


	//LEVEL DATA
	TCHAR szLevel[32];
	int Level;
	int LevelLeftBound;
	int LevelRightBound;
	int LevelTopBound;
	int LevelBottomBound;


	//OK AI TIME!!!! Handle AI in 3 states, defense, offense, GetBomb
	//If p1 has the bomb AI will be in defense
	//If AI has the bomb it will be in offense
	//During beginning of round AI will try to GetBomb
	//AI will shoot if the player is in range
	bool AIOffense;
	bool AIDefense;
	bool AIGetBomb;

	bool AIPlantBomb;
	bool AIDefuseBomb;


	struct Bullet
	{
	/*	HDC hdcBullets;
		HBITMAP hbmBullets;
		HBITMAP hbmBulletsOld;
		BITMAP bmBullets;*/

		int x1;
		int y1;
		int x2;
		int y2;

		int speedx;
		int speedy;

		RECT bulletBox;

	};


	// Variables used for display the Explosion animation
		HDC hdcExplosion;
		HBITMAP hbmExplosion;
		HBITMAP hbmExplosionOld;
		BITMAP bmExplosionInfo;

	struct Explosion
	{
		int ExplosionX;
		int ExplosionY;
		signed int RadiusX;
		signed int RadiusY;
		int Frame;
	};

	int frame;

	////////////////////////////////////
	std::vector<Explosion> ExplosionArray;

	std::vector<Bullet> Player1BulletArray;
	std::vector<Bullet> Player2BulletArray;

	bool P1P2Intersect;
public:
	volatile BOOL bRunning;
	bool AIEnabled;
	
	bool bMuteSoundFX;
	bool bMuteMusic;


	int selection;

	bool FirstPass;

	Game(void);
	virtual ~Game(void);

	//void TitleScreen(void);

	void ResetGame(int &Player1Score, int &Player2Score);
	void Startup(void);
	void Shutdown(void);

	void GetInput(void);
	void Simulate(void);
	void Render(void);

	void CreateExplosion(void);
	void Player1CreateBullet(void);
	void Player2CreateBullet(void);

	void DrawExplosion(void);
	void DrawBullet(void);

	void FreeExplosion();//int i);
	void Player1FreeBullet(int i);
	void Player2FreeBullet(int i);

	HDC getMemoryDC()
	{
		return hdcMemory;
	}
	void SetHwnd(HWND _hWnd) {hWnd = _hWnd;}

	void SetScores(int num1, int num2)
	{
		Player1Score = num1;
		Player2Score = num2;
	}

	bool MuteMusic()
	{
		return bMuteMusic;
	}

	bool MuteFX()
	{
		return bMuteSoundFX;
	}

	void ToggleFX()
	{
		bMuteSoundFX = !bMuteSoundFX;

		if(bMuteSoundFX)
				channel->setMute(true);
		else
				channel->setMute(false);

	}
	void ToggleMusic()
	{
		bMuteMusic = !bMuteMusic;

		if(bMuteMusic)
				musicChannel->setMute(true);
		else
				musicChannel->setMute(false);

	}

	int GetP1Score()
	{
		return Player1Score;
	}
	int GetP2Score()
	{
		return Player2Score;
	}

	void NewGame()
	{
		Player1Score = 0;
		Player2Score = 0;
		ResetGame(Player1Score,Player2Score);
	}
};